@@ -1,6 +1,7 @@
 "#error Do not run app-defaults files through xrdb!",
 "#error That does not do what you might expect.",
 "#error Put this file in /usr/lib/X11/app-defaults/XScreenSaver instead.",
+"*mode:			random",
 "*timeout:		0:10:00",
 "*cycle:			0:10:00",
 "*lockTimeout:		0:00:00",
@@ -11,8 +12,8 @@
 "*dpmsOff:		4:00:00",
 "*grabDesktopImages:	True",
 "*grabVideoFrames:	False",
-"*chooseRandomImages:	False",
-"*imageDirectory:	",
+"*chooseRandomImages:	True",
+"*imageDirectory:	/Library/Desktop Pictures/",
 "*nice:			10",
 "*memoryLimit:		0",
 "*lock:			False",
@@ -28,20 +29,25 @@
 "*visualID:		default",
 "*captureStderr: 	True",
 "*ignoreUninstalledPrograms: False",
+"*textMode:		file",
+"*textLiteral:		XScreenSaver",
+"*textFile:		/usr/X11R6/README",
+"*textProgram:		fortune",
+"*textURL:		http://www.livejournal.com/stats/latest-rss.bml",
 "*overlayTextForeground:	#FFFF00",
 "*overlayTextBackground:	#000000",
 "*overlayStderr:		True",
 "*font:			*-medium-r-*-140-*-m-*",
 "*sgiSaverExtension:	True",
-"*mitSaverExtension:	False",
 "*xidleExtension:	True",
 "*procInterrupts:	True",
 "GetViewPortIsFullOfLies: False",
 "*demoCommand: xscreensaver-demo",
 "*prefsCommand: xscreensaver-demo -prefs",
 "*helpURL: http://www.jwz.org/xscreensaver/man.html",
-"*loadURL: gnome-url-show '%s' || gnome-moz-remote --newwin '%s'",
-"*manualCommand: yelp 'man:%s' || gnome-help-browser 'man:%s'",
+"*loadURL: firefox '%s' || mozilla '%s' || netscape '%s'",
+"*manualCommand: xterm -sb -fg black -bg gray75 -T '%s manual' \
+		    -e /bin/sh -c 'man \"%s\" ; read foo'",
 "*dateFormat:		%d-%b-%y (%a); %I:%M %p",
 "*installColormap:	True",
 "*programs:								      \
@@ -207,8 +213,12 @@
 				anemotaxis -root			    \\n\
 				memscroller -root			    \\n\
 				substrate -root				    \\n\
+	 \"Substrate (circles)\"  substrate -root -circle-percent 33          \\n\
 				intermomentary -root			    \\n\
+				interaggregate -root			    \\n\
 				fireworkx -root				    \\n\
+				fiberlamp -root				    \\n\
+				boxfit -root				    \\n\
 - default-n:			webcollage -root			    \\n\
 - default-n:  \"WebCollage (whacked)\"					      \
 				webcollage -root -filter		      \
@@ -243,8 +253,7 @@
 	   GL:			starwars -root				    \\n\
 	   GL:			gltext -root				    \\n\
 	   GL:	\"GLText (clock)\" gltext -text \"%A%n%d %b %Y%n%r\" -root	    \\n\
-	   GL:	 \"Molecule\"		molecule -root			    \\n\
-	   GL:	 \"Molecule (lumpy)\"	molecule -root -no-bonds -no-labels \\n\
+	   GL:	 		molecule -root -shells			    \\n\
 	   GL:			dangerball -root			    \\n\
 	   GL:			circuit -root				    \\n\
 	   GL:			engine -root				    \\n\
@@ -285,8 +294,14 @@
 -	   GL:                  antinspect -root                            \\n\
 	   GL:			providence -root			    \\n\
 	   GL:	\"Pinion (large gears)\"	pinion -root			    \\n\
-	   GL:	\"Pinion (small gears)\"	pinion -root -size 0.2		    \\n\
-	   GL:			pinion -root				    \\n\
+	   GL:	\"Pinion (small gears)\"	pinion -root -size 0.2 -scroll 0.3  \\n\
+	   GL:			boing -root -lighting -smooth		    \\n\
+-	   GL:                  carousel -root                              \\n\
+	   GL:			fliptext -root				    \\n\
+-	   GL:                  antmaze -root                               \\n\
+	   GL:			tangram -root				    \\n\
+	   GL:			crackberg -root -flat -lit -crack	      \
+				 -color random				    \\n\
 									      \
 -				xdaliclock -root -builtin3 -cycle	    \\n\
 - default-n:			xearth -nofork -nostars -ncolors 50	      \
@@ -307,148 +322,43 @@
 -	   GL:                  sphereEversion --root                       \\n\
 -	   GL:                  fireflies -root                             \\n",
 "XScreenSaver.pointerPollTime:		0:00:05",
+"XScreenSaver.pointerHysteresis:		10",
 "XScreenSaver.initialDelay:		0:00:00",
 "XScreenSaver.windowCreationTimeout:	0:00:30",
 "XScreenSaver.bourneShell:		/bin/sh",
-"*Dialog.headingFont:		*-times-bold-r-*-*-*-180-*-*-*-iso8859-1",
+"*Dialog.headingFont:		*-helvetica-bold-r-*-*-*-180-*-*-*-iso8859-1",
 "*Dialog.bodyFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "*Dialog.labelFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
 "*Dialog.buttonFont:		*-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"*Dialog.dateFont:		*-courier-medium-r-*-*-*-80-*-*-*-iso8859-1",
+"*Dialog.dateFont:		*-helvetica-medium-r-*-*-*-80-*-*-*-iso8859-1",
+"*passwd.passwdFont:		*-courier-medium-r-*-*-*-140-*-*-*-iso8859-1",
 "*Dialog.foreground:		#000000",
-"*Dialog.background:		#D6D6D6",
+"*Dialog.background:		#E6E6E6",
 "*Dialog.Button.foreground:	#000000",
-"*Dialog.Button.background:	#EAEAEA",
+"*Dialog.Button.background:	#F5F5F5",
 "*Dialog.text.foreground:	#000000",
 "*Dialog.text.background:	#FFFFFF",
-"*passwd.thermometer.foreground:	#FF0000",
+"*passwd.thermometer.foreground:	#4464AC",
 "*passwd.thermometer.background:	#FFFFFF",
 "*Dialog.topShadowColor:		#FFFFFF",
-"*Dialog.bottomShadowColor:	#666666",
+"*Dialog.bottomShadowColor:	#CECECE",
 "*Dialog.logo.width:		210",
 "*Dialog.logo.height:		210",
-"*Dialog.internalBorderWidth:	30",
+"*Dialog.internalBorderWidth:	24",
 "*Dialog.borderWidth:		1",
 "*Dialog.shadowThickness:	2",
 "*passwd.heading.label:		XScreenSaver %s",
-"*passwd.body.label:		This display is locked.",
-"*passwd.user.label:		User:",
+"*passwd.body.label:		Please enter your password.",
+"*passwd.login.label:		New Login",
+"*passwd.user.label:		Username:",
 "*passwd.passwd.label:		Password:",
-"*passwd.passwdFont:		*-courier-medium-r-*-*-*-140-*-*-*-iso8859-1",
 "*passwd.thermometer.width:	8",
+"*passwd.asterisks:              True",
 "*splash.heading.label:		XScreenSaver %s",
-"*splash.body.label:		Copyright  1991-2004 by",
+"*splash.body.label:		Copyright  1991-2005 by",
 "*splash.body2.label:		Jamie Zawinski <jwz@jwz.org>",
 "*splash.demo.label:		Settings",
 "*splash.help.label:		Help",
-"*fontList:                       *-helvetica-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"*demoDialog*label1.fontList:     *-helvetica-medium-r-*-*-*-140-*-*-*-iso8859-1",
-"*cmdText.fontList:                 *-courier-medium-r-*-*-*-120-*-*-*-iso8859-1",
-"*label0.fontList:                  *-helvetica-bold-r-*-*-*-140-*-*-*-iso8859-1",
-"XScreenSaver*doc.fontList:       *-helvetica-medium-r-*-*-*-100-*-*-*-iso8859-1",
-"*foreground:			#000000",
-"*background:			#C0C0C0",
-"*XmTextField.foreground:	#000000",
-"*XmTextField.background:	#FFFFFF",
-"*list.foreground:		#000000",
-"*list.background:		#FFFFFF",
-"*ApplicationShell.title:	XScreenSaver",
-"*warning.title:			XScreenSaver",
-"*warning_popup.title:		XScreenSaver",
-"*allowShellResize:		True",
-"*autoUnmanage:			False",
-"*menubar*file.labelString:	File",
-"*menubar*file.mnemonic:		F",
-"*file.blank.labelString:	Blank Screen Now",
-"*file.blank.mnemonic:		B",
-"*file.lock.labelString:		Lock Screen Now",
-"*file.lock.mnemonic:		L",
-"*file.kill.labelString:		Kill Daemon",
-"*file.kill.mnemonic:		K",
-"*file.restart.labelString:	Restart Daemon",
-"*file.restart.mnemonic:		R",
-"*file.exit.labelString:		Exit",
-"*file.exit.mnemonic:		E",
-"*menubar*edit.labelString:	Edit",
-"*menubar*edit.mnemonic:		E",
-"*edit.cut.labelString:		Cut",
-"*edit.cut.mnemonic:		u",
-"*edit.copy.labelString:		Copy",
-"*edit.copy.mnemonic:		C",
-"*edit.paste.labelString:	Paste",
-"*edit.paste.mnemonic:		P",
-"*menubar*help.labelString:	Help",
-"*menubar*help.mnemonic:		H",
-"*help.about.labelString:	About...",
-"*help.about.mnemonic:		A",
-"*help.docMenu.labelString:	Documentation...",
-"*help.docMenu.mnemonic:		D",
-"*demoTab.marginWidth:		10",
-"*optionsTab.marginWidth:	10",
-"*XmScrolledWindow.topOffset:	10",
-"*XmScrolledWindow.leftOffset:	10",
-"*demoTab.topOffset:		4",
-"*form1.bottomOffset:		10",
-"*form3.leftOffset:		10",
-"*form3.rightOffset:		10",
-"*frame.topOffset:		10",
-"*frame.bottomOffset:		10",
-"*enabled.topOffset:		10",
-"*visLabel.topOffset:		10",
-"*combo.topOffset:		10",
-"*form4.bottomOffset:		4",
-"*hr.bottomOffset:		4",
-"*XmComboBox.marginWidth:	0",
-"*XmComboBox.marginHeight:	0",
-"*demo.marginWidth:		30",
-"*demo.marginHeight:		4",
-"*man.marginWidth:		10",
-"*man.marginHeight:		4",
-"*down.leftOffset:		40",
-"*down.marginWidth:		4",
-"*down.marginHeight:		4",
-"*up.marginWidth:		4",
-"*up.marginHeight:		4",
-"*frame.traversalOn:		False",
-"*list.automaticSelection:	True",
-"*list.visibleItemCount:		20",
-"*doc.columns:			60",
-"*combo.columns:			11",
-"*demoTab.labelString:		Graphics Demos",
-"*optionsTab.labelString:	Screensaver Options",
-"*down.labelString:		\\\\/ ",
-"*up.labelString:		/\\\\ ",
-"*frameLabel.labelString:	",
-"*cmdLabel.labelString:		Command Line:",
-"*cmdLabel.alignment:		ALIGNMENT_BEGINNING",
-"*enabled.labelString:		Enabled",
-"*visLabel.labelString:		Visual:",
-"*visLabel.alignment:		ALIGNMENT_END",
-"*visLabel.leftOffset:		20",
-"*demo.labelString:		Demo",
-"*man.labelString:		Documentation...",
-"*done.labelString:		Quit",
-"*preferencesLabel.labelString:	XScreenSaver Parameters",
-"*timeoutLabel.labelString:	Saver Timeout",
-"*cycleLabel.labelString:	Cycle Timeout",
-"*fadeSecondsLabel.labelString:	Fade Duration",
-"*fadeTicksLabel.labelString:	Fade Ticks",
-"*lockLabel.labelString:		Lock Timeout",
-"*passwdLabel.labelString:	Password Timeout",
-"*preferencesForm*XmTextField.columns:	8",
-"*verboseToggle.labelString:	Verbose",
-"*cmapToggle.labelString:	Install Colormap",
-"*fadeToggle.labelString:	Fade Colormap",
-"*unfadeToggle.labelString:	Unfade Colormap",
-"*lockToggle.labelString:	Require Password",
-"*OK.marginWidth:		30",
-"*OK.marginHeight:		4",
-"*OK.leftOffset:			10",
-"*OK.bottomOffset:		10",
-"*Cancel.marginWidth:		30",
-"*Cancel.marginHeight:		4",
-"*Cancel.rightOffset:		10",
-"*Cancel.bottomOffset:		10",
 "*hacks.imsmap.name:         IMSmap",
 "*hacks.slidescreen.name:    SlideScreen",
 "*hacks.decayscreen.name:    DecayScreen",
@@ -502,4 +412,6 @@
 "*hacks.blinkbox.name:       BlinkBox",
 "*hacks.fuzzyflakes.name:    FuzzyFlakes",
 "*hacks.memscroller.name:    MemScroller",
+"*hacks.boxfit.name:         BoxFit",
+"*hacks.fliptext.name:       FlipText",
 "*hacks.documentation.isInstalled: True",
