@@ -57,7 +57,7 @@ static char *usage = "\n\
 usage: %s -<option>\n\
 \n\
   This program provides external control of a running xscreensaver process.\n\
-  Version %s, copyright (c) 1991-2003 Jamie Zawinski <jwz@jwz.org>.\n\
+  Version %s, copyright (c) 1991-2005 Jamie Zawinski <jwz@jwz.org>.\n\
 \n\
   The xscreensaver program is a daemon that runs in the background.\n\
   You control a running xscreensaver process by sending it messages\n\
@@ -95,9 +95,8 @@ usage: %s -<option>\n\
                 with a particular graphics demo.  (The first element in the\n\
                 list is numbered 1, not 0.)\n\
 \n\
-  -exit         Causes the xscreensaver process to exit gracefully.  This is\n\
-                roughly the same as killing the process with `kill', but it\n\
-                is easier, since you don't need to first figure out the pid.\n\
+  -exit         Causes the xscreensaver process to exit gracefully.\n\
+                This does nothing if the display is currently locked.\n\
                 (Note that one must *never* kill xscreensaver with -9!)\n\
 \n\
   -restart      Causes the screensaver process to exit and then restart with\n\
