@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* crystal --- polygons moving according to plane group rules */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)crystal.c	4.12 98/09/10 xlockmore";
-
 #endif
 
 /*-
@@ -109,29 +108,29 @@ static Bool unit_cell, grid_cell, centre, maxsize, cycle_p;
 
 static XrmOptionDescRec opts[] =
 {
-	{"-nx", "crystal.nx", XrmoptionSepArg, (caddr_t) NULL},
-	{"-ny", "crystal.ny", XrmoptionSepArg, (caddr_t) NULL},
-	{"-centre", ".crystal.centre", XrmoptionNoArg, (caddr_t) "on"},
-	{"+centre", ".crystal.centre", XrmoptionNoArg, (caddr_t) "off"},
-	{"-maxsize", ".crystal.maxsize", XrmoptionNoArg, (caddr_t) "on"},
-	{"+maxsize", ".crystal.maxsize", XrmoptionNoArg, (caddr_t) "off"},
-	{"-cell", ".crystal.cell", XrmoptionNoArg, (caddr_t) "on"},
-	{"+cell", ".crystal.cell", XrmoptionNoArg, (caddr_t) "off"},
-	{"-grid", ".crystal.grid", XrmoptionNoArg, (caddr_t) "on"},
-	{"+grid", ".crystal.grid", XrmoptionNoArg, (caddr_t) "off"},
-	{"-shift", ".crystal.shift", XrmoptionNoArg, (caddr_t) "on"},
-	{"+shift", ".crystal.shift", XrmoptionNoArg, (caddr_t) "off"}
+	{"-nx", "crystal.nx", XrmoptionSepArg, 0},
+	{"-ny", "crystal.ny", XrmoptionSepArg, 0},
+	{"-centre", ".crystal.centre", XrmoptionNoArg, "on"},
+	{"+centre", ".crystal.centre", XrmoptionNoArg, "off"},
+	{"-maxsize", ".crystal.maxsize", XrmoptionNoArg, "on"},
+	{"+maxsize", ".crystal.maxsize", XrmoptionNoArg, "off"},
+	{"-cell", ".crystal.cell", XrmoptionNoArg, "on"},
+	{"+cell", ".crystal.cell", XrmoptionNoArg, "off"},
+	{"-grid", ".crystal.grid", XrmoptionNoArg, "on"},
+	{"+grid", ".crystal.grid", XrmoptionNoArg, "off"},
+	{"-shift", ".crystal.shift", XrmoptionNoArg, "on"},
+	{"+shift", ".crystal.shift", XrmoptionNoArg, "off"}
 };
 
 static argtype vars[] =
 {
-	{(caddr_t *) & nx, "nx", "nx", DEF_NX, t_Int},
-	{(caddr_t *) & ny, "ny", "ny", DEF_NY, t_Int},
-	{(caddr_t *) & centre, "centre", "Centre", DEF_CENTRE, t_Bool},
-	{(caddr_t *) & maxsize, "maxsize", "Maxsize", DEF_MAXSIZE, t_Bool},
-	{(caddr_t *) & unit_cell, "cell", "Cell", DEF_CELL, t_Bool},
-	{(caddr_t *) & grid_cell, "grid", "Grid", DEF_GRID, t_Bool},
-	{(caddr_t *) & cycle_p, "shift", "Shift", DEF_CYCLE, t_Bool}
+	{&nx, "nx", "nx", DEF_NX, t_Int},
+	{&ny, "ny", "ny", DEF_NY, t_Int},
+	{&centre, "centre", "Centre", DEF_CENTRE, t_Bool},
+	{&maxsize, "maxsize", "Maxsize", DEF_MAXSIZE, t_Bool},
+	{&unit_cell, "cell", "Cell", DEF_CELL, t_Bool},
+	{&grid_cell, "grid", "Grid", DEF_GRID, t_Bool},
+	{&cycle_p, "shift", "Shift", DEF_CYCLE, t_Bool}
 };
 static OptionStruct desc[] =
 {
