@@ -8,7 +8,7 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
 #endif
 
 /*-
- * Copyright (c) 1996 by Tim Auckland <Tim.Auckland@Procket.com>
+ * Copyright (c) 1996 by Tim Auckland <tda10.geo@yahoo.com>
  * Incorporating some code from Stephen Davies Copyright (c) 2000
  *
  * Search code based on techniques described in "Strange Attractors:
@@ -107,12 +107,7 @@ static const char sccsid[] = "@(#)flow.c	5.00 2000/11/01 xlockmore";
 					"*size:         -10 \n" \
 					"*cycles:       10000 \n" \
 					"*ncolors:      200 \n" \
-					"*rotate:       True \n" \
-					"*ride:         True \n" \
-					"*box:          True \n" \
-					"*periodic:     True \n" \
-					"*search:       True \n" \
-					"*dbuf:         True \n"
+
 # include "xlockmore.h"		/* in xscreensaver distribution */
 # ifndef MI_DEPTH
 #  define MI_DEPTH MI_WIN_DEPTH
