@@ -46,10 +46,6 @@ extern XtAppContext app;
 #define DEFAULTS	"*delay:	30000       \n" \
 			"*showFPS:      False       \n" \
 			"*wireframe:    False       \n" \
-			"*speed:      " DEF_SPEED " \n" \
-			"*ntoasters:  " DEF_NTOASTERS "\n" \
-			"*nslices:    " DEF_NSLICES   "\n" \
-			"*texture:    " DEF_TEXTURE   "\n" \
 
 /* #define DEBUG */
 
@@ -324,7 +320,7 @@ toaster_handle_event (ModeInfo *mi, XEvent *event)
   toaster_configuration *bp = &bps[MI_SCREEN(mi)];
 
   if (event->xany.type == ButtonPress &&
-      event->xbutton.button & Button1)
+      event->xbutton.button == Button1)
     {
       bp->button_down_p = True;
       gltrackball_start (bp->user_trackball,
@@ -333,11 +329,19 @@ toaster_handle_event (ModeInfo *mi, XEvent *event)
       return True;
     }
   else if (event->xany.type == ButtonRelease &&
-           event->xbutton.button & Button1)
+           event->xbutton.button == Button1)
     {
       bp->button_down_p = False;
       return True;
     }
+  else if (event->xany.type == ButtonPress &&
+           (event->xbutton.button == Button4 ||
+            event->xbutton.button == Button5))
+    {
+      gltrackball_mousewheel (bp->user_trackball, event->xbutton.button, 5,
+                              !event->xbutton.state);
+      return True;
+    }
   else if (event->xany.type == MotionNotify &&
            bp->button_down_p)
     {
