@@ -45,14 +45,7 @@
 # define planet_opts					xlockmore_opts
 #define DEFAULTS	"*delay:			20000   \n"	\
 					"*showFPS:			False   \n" \
-                    "*rotate:           True    \n" \
-                    "*roll:             True    \n" \
-                    "*wander:           True    \n" \
 					"*wireframe:		False	\n"	\
-					"*light:			True	\n"	\
-					"*texture:			True	\n" \
-					"*stars:			True	\n" \
-					"*image:			BUILTIN	\n" \
 					"*imageForeground:	Green	\n" \
 					"*imageBackground:	Blue	\n"
 
@@ -183,25 +176,6 @@ typedef struct {
 static planetstruct *planets = NULL;
 
 
-static inline void
-normalize(GLfloat v[3])
-{
-  GLfloat d = (GLfloat) sqrt((double) (v[0] * v[0] +
-                                       v[1] * v[1] +
-                                       v[2] * v[2]));
-  if (d != 0)
-    {
-      v[0] /= d;
-      v[1] /= d;
-      v[2] /= d;
-	}
-  else
-    {
-      v[0] = v[1] = v[2] = 0;
-	}
-}
-
-
 /* Set up and enable texturing on our object */
 static void
 setup_xpm_texture (ModeInfo *mi, char **xpm_data)
@@ -467,7 +441,7 @@ planet_handle_event (ModeInfo *mi, XEvent *event)
   planetstruct *gp = &planets[MI_SCREEN(mi)];
 
   if (event->xany.type == ButtonPress &&
-      event->xbutton.button & Button1)
+      event->xbutton.button == Button1)
     {
       gp->button_down_p = True;
       gltrackball_start (gp->trackball,
@@ -476,11 +450,19 @@ planet_handle_event (ModeInfo *mi, XEvent *event)
       return True;
     }
   else if (event->xany.type == ButtonRelease &&
-           event->xbutton.button & Button1)
+           event->xbutton.button == Button1)
     {
       gp->button_down_p = False;
       return True;
     }
+  else if (event->xany.type == ButtonPress &&
+           (event->xbutton.button == Button4 ||
+            event->xbutton.button == Button5))
+    {
+      gltrackball_mousewheel (gp->trackball, event->xbutton.button, 10,
+                              !!event->xbutton.state);
+      return True;
+    }
   else if (event->xany.type == MotionNotify &&
            gp->button_down_p)
     {
