@@ -110,18 +110,7 @@ static const char sccsid[] = "@(#)hypertorus.c  1.1 03/05/18 xlockmore";
 # define hypertorus_opts xlockmore_opts
 # define DEFAULTS        "*delay:      25000 \n" \
                          "*showFPS:    False \n" \
-                         "*wireframe:  False \n" \
-                         "*displayMode: "  DEF_DISPLAY_MODE  " \n" \
-                         "*appearance: "   DEF_APPEARANCE    " \n" \
-                         "*colors: "       DEF_COLORS        " \n" \
-                         "*projection3d: " DEF_3D_PROJECTION " \n" \
-                         "*projection4d: " DEF_4D_PROJECTION " \n" \
-                         "speedwx: "       DEF_DALPHA        " \n" \
-                         "speedwy: "       DEF_DBETA         " \n" \
-                         "speedwz: "       DEF_DDELTA        " \n" \
-                         "speedxy: "       DEF_DZETA         " \n" \
-                         "speedxz: "       DEF_DETA          " \n" \
-                         "speedyz: "       DEF_DTHETA        " \n"
+
 # include "xlockmore.h"         /* from the xscreensaver distribution */
 #else  /* !STANDALONE */
 # include "xlock.h"             /* from the xlockmore distribution */
@@ -170,7 +159,7 @@ static const float offset3d[4] = {
 
 static XrmOptionDescRec opts[] =
 {
-  {"-mesh",            ".hypertorus.displayMode",  XrmoptionNoArg,
+  {"-wireframe",       ".hypertorus.displayMode",  XrmoptionNoArg,
                        DISP_WIREFRAME_STR },
   {"-surface",         ".hypertorus.displayMode",  XrmoptionNoArg,
                        DISP_SURFACE_STR },
@@ -228,7 +217,7 @@ static argtype vars[] =
 
 static OptionStruct desc[] =
 {
-  { "-mesh",            "display the torus as a wireframe mesh" },
+  { "-wireframe",       "display the torus as a wireframe mesh" },
   { "-surface",         "display the torus as a solid surface" },
   { "-transparent",     "display the torus as a transparent surface" },
   { "-solid",           "display the torus as a solid object" },
