@@ -1,9 +1,8 @@
 /* -*- Mode: C; tab-width: 4 -*- */
 /* morph3d --- Shows 3D morphing objects */
 
-#if !defined( lint ) && !defined( SABER )
+#if 0
 static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
-
 #endif
 
 #undef DEBUG_CULL_FACE
@@ -52,15 +51,6 @@ static const char sccsid[] = "@(#)morph3d.c	5.01 2001/03/01 xlockmore";
  *
  */
 
-#ifdef VMS
-/*-
- * due to a Bug/feature in VMS X11/Intrinsic.h has to be placed before xlock.
- * otherwise caddr_t is not defined correctly
- */
-
-#include <X11/Intrinsic.h>
-#endif
-
 #ifdef STANDALONE
 # define MODE_moebius
 # define PROGCLASS		"Morph3d"
