@@ -1377,7 +1377,9 @@ find_dead_regions(void)
 	  if((x < logo_x || x > logo_x + logo_width / grid_width) ||
 	     (y < logo_y || y > logo_y + logo_height / grid_height))
 	  {
-	    if (!(maze[x][y] & WALL_ANY))
+            /* if we are completely surrounded by walls, just draw the
+               inside part */
+            if ((maze[x][y] & WALL_ANY) == WALL_ANY)
 	      XFillRectangle(dpy, win, ugc,
 			     border_x + bw + grid_width * x,
 			     border_y + bw + grid_height * y,
@@ -1659,8 +1661,7 @@ screenhack(Display *display, Window window)
       XWindowAttributes xgwa;
       XGetWindowAttributes (dpy, window, &xgwa);
       XSelectInput (dpy, win,
-                    xgwa.your_event_mask | ExposureMask |
-                    ButtonPressMask |StructureNotifyMask);
+                    xgwa.your_event_mask | ExposureMask | ButtonPressMask);
     }
   
   gc  = XCreateGC(dpy, win, 0, 0);
